import com.google.gson.Gson;
import net.sf.json.JSONArray;
import net.sf.json.JSONObject;
import okhttp3.*;
import org.apache.commons.codec.binary.Hex;
import org.apache.commons.codec.digest.DigestUtils;
import org.bouncycastle.jce.provider.BouncyCastleProvider;

import javax.crypto.Cipher;
import java.io.*;
import java.nio.charset.StandardCharsets;
import java.security.PublicKey;
import java.security.Security;
import java.security.cert.CertificateFactory;
import java.security.cert.X509Certificate;
import java.util.Objects;

/**
 * @author colin, pengchen
 * @since 2013-5-9 上午11:47:03
 *
 * 本范例代码，可以通过修改参数，运行main函数，查看接口效果
 *
 * 以下是本范例代码需要的maven依赖配置
    <dependency>
        <groupId>com.google.code.gson</groupId>
        <artifactId>gson</artifactId>
        <version>2.8.6</version>
    </dependency>
    <dependency>
        <groupId>net.sf.json-lib</groupId>
        <artifactId>json-lib</artifactId>
        <version>2.2.3</version>
    </dependency>
    <dependency>
        <groupId>commons-codec</groupId>
        <artifactId>commons-codec</artifactId>
        <version>1.4</version>
    </dependency>
    <dependency>
        <groupId>org.bouncycastle</groupId>
        <artifactId>bcprov-jdk15on</artifactId>
        <version>1.62</version>
    </dependency>
    <dependency>
        <groupId>com.squareup.okhttp3</groupId>
        <artifactId>okhttp</artifactId>
        <version>4.9.1</version>
    </dependency>
 */
public class Example {
    /**
     * 自动发布接口域名
     */
    private static final String DOMAIN = "http://api.developer.xiaomi.com/devupload";

    /**
     * 推送普通apk Url前缀
     */
    private static final String PUSH = DOMAIN + "/dev/push";
    /**
     * 推送渠道包Url前缀
     */
    private static final String PUSH_CHANNEL_APK = DOMAIN + "/dev/pushChannelApk";
    /**
     * 查询app状态的Url前缀
     */
    private static final String QUERY = DOMAIN + "/dev/query";
    /**
     * 查询应用分类Url前缀
     */
    private static final String CATEGORY = DOMAIN + "/dev/category";

    /**
     * 以下四项为接口参数加密算法X509用到的参数
     */
    private static final int KEY_SIZE = 1024;
    private static final int GROUP_SIZE = KEY_SIZE / 8;
    private static final int ENCRYPT_GROUP_SIZE = GROUP_SIZE - 11;
    public static final String KEY_ALGORITHM = "RSA/NONE/PKCS1Padding";

    private Gson Gson = new Gson();

    private static PublicKey pubKey;

    /**
     * 加载BC库
     */
    static {
        Security.addProvider(new BouncyCastleProvider());
    }

    /**
     * 执行具体的接口前，需要加载公钥证书
     */
    static {
        try {
            pubKey = getPublicKeyByX509Cer("本地公钥路径");
        } catch (Exception e) {
            e.printStackTrace();
        }
    }

    /**
     * 读取公钥
     *
     * @param cerFilePath 本地公钥存放的文件目录
     * @return 返回公钥
     * @throws Exception
     */
    public static PublicKey getPublicKeyByX509Cer(String cerFilePath) throws Exception {
        try (InputStream x509Is = new FileInputStream(cerFilePath)) {
            CertificateFactory certificatefactory = CertificateFactory.getInstance("X.509");
            X509Certificate cert = (X509Certificate) certificatefactory.generateCertificate(x509Is);
            return cert.getPublicKey();
        } catch (Exception e) {
            e.printStackTrace();
            throw e;
        }
    }

    /**
     * 使用公钥加密
     *
     * @param str
     * @param publicKey
     * @return
     * @throws Exception
     */
    public static String encryptByPublicKey(String str, PublicKey publicKey) throws Exception {
        byte[] data = str.getBytes();
        ByteArrayOutputStream baos = new ByteArrayOutputStream();
        byte[] segment = new byte[ENCRYPT_GROUP_SIZE];
        int idx = 0;
        Cipher cipher = Cipher.getInstance(KEY_ALGORITHM, "BC");
        cipher.init(Cipher.ENCRYPT_MODE, publicKey);
        while (idx < data.length) {
            int remain = data.length - idx;
            int segsize = Math.min(remain, ENCRYPT_GROUP_SIZE);
            System.arraycopy(data, idx, segment, 0, segsize);
            baos.write(cipher.doFinal(segment, 0, segsize));
            idx += segsize;
        }
        return Hex.encodeHexString(baos.toByteArray());
    }

    /*
     * 获取文件md5
     */
    private static String getFileMD5(File file) throws Exception {
        try (FileInputStream fis = new FileInputStream(file)) {
            return DigestUtils.md5Hex(fis);
        } catch (Exception e) {
            e.printStackTrace();
            throw e;
        }
    }

    /**
     * 查询应用分类
     *
     * @return
     */
    public String getCatetories() {
        OkHttpClient httpClient = new OkHttpClient();
        RequestBody requestBody = new FormBody.Builder().build();
        Request request = new Request.Builder().url(CATEGORY).post(requestBody).build();
        try {
            return Objects.requireNonNull(httpClient.newCall(request).execute().body()).string();
        } catch (IOException e) {
            e.printStackTrace();
        }
        return "";
    }

    /**
     * 查询应用信息，前提是小米开放平台已经创建了包名
     *
     * @param userName
     * @param password
     * @param packageName 包名
     * @return
     */
    public String query(String userName, String password, String packageName) {
        OkHttpClient httpClient = new OkHttpClient();
        JSONObject json = new JSONObject();
        json.put("packageName", packageName);
        json.put("userName", userName);
        FormBody.Builder formBodyBuilder = new FormBody.Builder();
        formBodyBuilder.add("RequestData", json.toString());

        JSONObject sig = new JSONObject();
        JSONArray sigs = new JSONArray();

        JSONObject sigItem = new JSONObject();
        sigItem.put("name", "RequestData");
        sigItem.put("hash", DigestUtils.md5Hex(json.toString()));

        sigs.add(sigItem);
        sig.put("sig", sigs);
        sig.put("password", password);

        try {
            RequestBody requestBody = formBodyBuilder.add("SIG", encryptByPublicKey(sig.toString(), pubKey)).build();

            Request request = new Request.Builder().url(QUERY).post(requestBody).build();
            Response response = httpClient.newCall(request).execute();
            return Objects.requireNonNull(response.body()).string();
        } catch (Exception ex) {
            ex.printStackTrace();
        }
        return "";
    }

    /**
     * 推送应用
     *
     * @param email
     * @param password
     * @param synchroType 0：新增app; 1：更新app; 2：app信息修改
     * @param apkFile
     * @param iconFile
     * @param appDetail
     * @param screenshots
     * @return
     * @throws Exception
     */
    public String push(String email, String password, int synchroType, File apkFile, File iconFile, AppDetailBean appDetail,
                       File... screenshots) throws Exception {
        OkHttpClient httpClient = new OkHttpClient();
        JSONObject json = new JSONObject();
        json.put("userName", email);
        json.put("appInfo", Gson.toJson(appDetail));
        json.put("synchroType", synchroType);

        // 计算数字签名
        JSONObject sigJSON = new JSONObject();
        JSONArray paramMd5Array = new JSONArray();
        JSONObject sigItem = new JSONObject();
        sigItem.put("name", "RequestData");
        sigItem.put("hash", DigestUtils.md5Hex(json.toString()));

        paramMd5Array.add(sigItem);

        MultipartBody.Builder builder = new MultipartBody.Builder();

        if (apkFile != null) {
            JSONObject apk = new JSONObject();
            apk.put("name", "apk");
            apk.put("hash", getFileMD5(apkFile));
            paramMd5Array.add(apk);
            builder.addFormDataPart("apk", "",
                    RequestBody.create(apkFile, MediaType.parse("application/octet-stream")));
        }

        if (iconFile != null) {
            JSONObject icon = new JSONObject();
            icon.put("name", "icon");
            icon.put("hash", getFileMD5(iconFile));
            paramMd5Array.add(icon);
            builder.addFormDataPart("icon", "",
                    RequestBody.create(iconFile, MediaType.parse("application/octet-stream")));
        }

        if (screenshots != null && screenshots.length > 0) {
            for (int i = 0; i < screenshots.length; i++) {
                String screenshotName = "screenshot_" + (i + 1);
                JSONObject screenshot = new JSONObject();
                screenshot.put("name", screenshotName);
                screenshot.put("hash", getFileMD5(screenshots[i]));
                paramMd5Array.add(screenshot);

                builder.addFormDataPart(screenshotName, "",
                        RequestBody.create(screenshots[i], MediaType.parse("application/octet-stream")));
            }
        }
        sigJSON.put("sig", paramMd5Array);
        sigJSON.put("password", password);
        builder.addFormDataPart("RequestData", new String(json.toString().getBytes(), StandardCharsets.UTF_8));
        builder.addFormDataPart("SIG", new String(encryptByPublicKey(sigJSON.toString(), pubKey).getBytes(), StandardCharsets.UTF_8));
        RequestBody requestBody = builder.build();

        try {
            Request request = new Request.Builder().url(PUSH).post(requestBody).build();
            Response response = httpClient.newCall(request).execute();
            return Objects.requireNonNull(response.body()).string();
        } catch (Exception e) {
            e.printStackTrace();
            return "";
        }
    }

    /*
     * 推送渠道包
     *
     * @param email
     * @param password
     * @param channelApkFile
     */
    private String pushChannelApk(String email, String password, String apkChannel, File channelApkFile) throws Exception {
        OkHttpClient httpClient = new OkHttpClient();

        JSONObject json = new JSONObject();
        json.put("userName", email);
        json.put("apkChannel", apkChannel);

        // 计算数字签名
        JSONArray paramMd5Array = new JSONArray();
        JSONObject sigItem = new JSONObject();
        sigItem.put("name", "RequestData");
        sigItem.put("hash", DigestUtils.md5Hex(json.toString()));
        paramMd5Array.add(sigItem);

        MultipartBody.Builder builder = new MultipartBody.Builder();

        if (channelApkFile != null) {
            JSONObject apk = new JSONObject();
            apk.put("name", "channelApk");
            apk.put("hash", getFileMD5(channelApkFile));
            paramMd5Array.add(apk);
            builder.addFormDataPart("channelApk", "",
                    RequestBody.create(channelApkFile, MediaType.parse("application/octet-stream")));
        }

        JSONObject sigJSON = new JSONObject();
        sigJSON.put("sig", paramMd5Array);
        sigJSON.put("password", password);
        builder.addFormDataPart("RequestData", new String(json.toString().getBytes(), StandardCharsets.UTF_8));
        builder.addFormDataPart("SIG", new String(encryptByPublicKey(sigJSON.toString(), pubKey).getBytes(), StandardCharsets.UTF_8));
        RequestBody requestBody = builder.build();
        try {
            Request request = new Request.Builder().url(PUSH_CHANNEL_APK).post(requestBody).build();
            Response response = httpClient.newCall(request).execute();
            return Objects.requireNonNull(response.body()).string();
        } catch (Exception e) {
            e.printStackTrace();
            return "";
        }
    }

    /**
     * app信息
     */
    public static class AppDetailBean {
        private String appName;
        private String packageName;
        private String publisherName;
        private int category;
        private String keyWords;
        private String versionName;
        private String desc;
        private String web;
        private String updateDesc;
        private String price;
        private String shortDesc;

        public String getPackageName() {
            return packageName;
        }

        public void setPackageName(String packageName) {
            this.packageName = packageName;
        }

        public String getAppName() {
            return appName;
        }

        public void setAppName(String appName) {
            this.appName = appName;
        }

        public String getPublisherName() {
            return publisherName;
        }

        public void setPublisherName(String publisherName) {
            this.publisherName = publisherName;
        }

        public int getCategory() {
            return category;
        }

        public void setCategory(int category) {
            this.category = category;
        }

        public String getKeyWords() {
            return keyWords;
        }

        public void setKeyWords(String keyWords) {
            this.keyWords = keyWords;
        }

        public String getVersionName() {
            return versionName;
        }

        public void setVersionName(String versionName) {
            this.versionName = versionName;
        }

        public String getDesc() {
            return desc;
        }

        public void setDesc(String desc) {
            this.desc = desc;
        }

        public String getWeb() {
            return web;
        }

        public void setWeb(String web) {
            this.web = web;
        }

        public String getUpdateDesc() {
            return updateDesc;
        }

        public void setUpdateDesc(String updateDesc) {
            this.updateDesc = updateDesc;
        }

        public String getPrice() {
            return price;
        }

        public void setPrice(String price) {
            this.price = price;
        }

        public String getShortDesc() {
            return shortDesc;
        }

        public void setShortDesc(String shortDesc) {
            this.shortDesc = shortDesc;
        }
    }

    /**
     * 运行此函数，除了查询分类接口以外，其余接口都需要先创建应用包名
     * 推送渠道包，需要普通应用在架
     * 目前小米开放平台暂时没有开放自动发布接口测试环境，如果只是开发测试，推送应用成功后，
     * 可以在小米开放平台操作页面撤回审核，继续测试推送
     * @param args
     * @throws Exception
     */
    public static void main(String[] args) throws Exception {
        Example example = new Example();
        String email = "小米账号邮箱";
        String pwd = "小米账号密码或者自动发布接口私钥";

        // 查询分类
        String category = example.getCatetories();
        System.out.println(category);

        // 查询应用信息
        String info = example.query(email, pwd, "包名");
        System.out.println("详情[" + info + "]");

        // 推送应用
        AppDetailBean appDetail = new AppDetailBean();
        appDetail.setAppName("应用名称");
        appDetail.setCategory(2);
        appDetail.setDesc("应用详情");
        appDetail.setPackageName("包名");
        appDetail.setKeyWords("关键字1 关键字2");
        appDetail.setWeb("应用官网");

        File apkFile = new File("apk文件路径");
        File iconFile = new File("icon文件路径");

        File screenshot_1File = new File("截图1文件路径");
        File screenshot_2File = new File("截图2文件路径");
        File screenshot_3File = new File("截图3文件路径");

        File[] screenshots = {screenshot_1File, screenshot_2File, screenshot_3File};
        String result = example.push(email, pwd, 0, apkFile, iconFile, appDetail, screenshots);
        System.out.println("结果[" + result + "]");

        // 提交渠道包
        String apkChannel = "渠道名称";
        File channelApkFile = new File("渠道包文件路径");
        String channelApkResult = example.pushChannelApk(email, pwd, apkChannel, channelApkFile);
        System.out.println("结果[" + channelApkResult + "]");
    }
}
